DAT programName  byte "LoadToEepromSerial_150226c", 0
' I like to have the name of the program displayed in a terminal window.
CON{{
  Program to load PASM sections of "Servo32v9" and "F32" to EEPROM.

  There is also code to generate the a call table for "F32" which
  does not require being compiled with the PASM section of the code.
  
}}
CON{

  150206a Tighty up in preparation to post to the forum.
  06b Remove header file.
  26c Use updated serial driver.   
}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000 

  DEBUG_BAUD = 115_200

  SERIAL_EEPROM_START = $8000
  
  QUOTE = 34
  BELL = 7
  
OBJ

  Pst : "Parallax Serial Terminal" ' used by this program to display debug information                         
  Serial : "SerialToEeprom150226a" ' The four port serial driver being written to EEPROM

  Ee : "dwd_PropellerEeprom"
  
PUB Main | writeFlag, first, last, size

  Pst.Start(DEBUG_BAUD)                                            

  writeFlag := 0 ' change this to one to write to EEPROM
  
  repeat
    NewLine
    NewLine
    Pst.Str(string("Press any key to begin."))
    NewLine
    NewLine
    if writeFlag   
      Pst.Str(string("Warning, this program will write to the upper EEPROM."))
    else
      Pst.Str(string("This program will display information about writing to"))
      NewLine
      Pst.Str(string("upper EEPROM but it will not change the EEPROM contents."))
    waitcnt(clkfreq / 2 + cnt)
    result := Pst.RxCount    
  until result 

  Pst.Clear
  Pst.Home

  waitcnt(clkfreq / 20 + cnt)
 
  NewLine
  Pst.Str(string("Serial PASM Starts (RAM) $"))
  first := Serial.GetFirst
  Pst.Hex(first, 4)
  NewLine
  Pst.Str(string("Serial PASM Last Long (RAM) $"))
  last := Serial.GetLast
  Pst.Hex(last, 4)
  NewLine
  Pst.Str(string("Serial PASM Size $"))
  size := (last - first) + 4 ' add the last long to the size
  Pst.Hex(size, 4)

  FromRam(first, last + 3, SERIAL_EEPROM_START, writeFlag)
                       '^ add the three bytes in the last long
  NewLine
  Pst.Str(string("Start of Serial PASM in EEPROM = $"))
  Pst.Hex(SERIAL_EEPROM_START, 4)
 
  NewLine
  Pst.Str(string("Start of unused area in EEPROM = $"))
  Pst.Hex(SERIAL_EEPROM_START + size , 4)

  NewLine
  Pst.Str(string("Write down the above EEPROM addresses to use in the program"))
  NewLine
  Pst.Str(string("which will read this PASM section."))
  NewLine
  NewLine
  Pst.Str(string("Press any key to compare EEPROM data with RAM data."))

  Pst.RxFlush
  
  repeat
    waitcnt(clkfreq / 4 + cnt)
    result := Pst.RxCount    
  until result

  result := CompareLongs(first, SERIAL_EEPROM_START, size / 4)

  NewLine
  NewLine
  if result and writeFlag
    Pst.Str(string("There was an error writing to EEPROM."))
  elseif result
    Pst.Str(string("The PASM data has not been written to EEPROM yet.")) 
    NewLine
    Pst.Str(string("Set ", QUOTE, "writeFlag", QUOTE, " to 1 and run program again to write data to EEPROM."))
  
  NewLine
  NewLine
  Pst.Str(string("End of program."))

  repeat

PUB FromRam(ramStart, ramStop, eepromStart, writeFlag)

  NewLine
  Pst.Str(string("Starts (RAM) $"))
  Pst.Hex(ramStart, 4)
  NewLine
  Pst.Str(string("Last (RAM) $"))
  Pst.Hex(ramStop, 4)
  NewLine
  Pst.Str(string("PASM Size $"))
  Pst.Hex((ramStop - ramStart) + 1, 4)
  NewLine
  Pst.Str(string("Ee.FromRam($"))
  Pst.Hex(ramStart, 4)
  Pst.Str(string(", $"))
  Pst.Hex(ramStop, 4)
  Pst.Str(string(", $"))
  Pst.Hex(eepromStart, 4)
  Pst.Char(")")
  if writeFlag
    Pst.Str(string(" *** Writing EEPROM ***"))
    Ee.FromRam(ramStart, ramStop, eepromStart)
  else
    Pst.Str(string(" Not Writing to EEPROM"))

PUB CompareLongs(localPtr, eepromPtr, localSize) | localLong, rowCount

  rowCount := 0
  NewLine
  Pst.Str(string("Comparing RAM with EEPROM."))
  NewLine
  Pst.Str(string("RAM address = $"))
  Pst.Hex(localPtr, 4)
  NewLine
  Pst.Str(string("EEPROM address = $"))
  Pst.Hex(eepromPtr, 4)
  NewLine
  Pst.Str(string("Longs to compare = $"))
  Pst.Hex(localSize, 4)
  NewLine
  
  repeat localSize
    ifnot rowCount++ // 4
      NewLine
      Pst.Str(string("<$"))
      Pst.Hex(localPtr, 4)
      Pst.Str(string("&$"))
      Pst.Hex(eepromPtr, 4)
      Pst.Str(string(">"))

    Ee.ToRam(@localLong, @localLong + 3, eepromPtr)
    Pst.Str(string("|$"))
    Pst.Hex(long[localPtr], 8)
    if long[localPtr] == localLong
      Pst.Str(string("==$"))
    else
      Pst.Str(string(7, "<>$"))
      result++
    Pst.Hex(localLong, 8)  
    localPtr += 4
    eepromPtr += 4

  NewLine
  Pst.Str(string("The compare method found "))
  Pst.Dec(result)
  Pst.Str(string(" unmatched longs."))
  NewLine 
  if result
    Pst.Str(string(BELL, " Failure!!! ", BELL))
  else
    Pst.Str(string(" Success!!! "))

PRI NewLine                                      
'' Clear the end of the current line and then
'' start a new line.

  Pst.ClearEnd
  Pst.Newline
  result := 1